<?php
/**
 * Created by PhpStorm.
 * User: Skalden
 * Date: 6/18/2018
 * Time: 10:55 AM
 */

namespace App\Http\Controllers;

use App\Models\Package;
use App\Models\PackageItems;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use Exception;

class PackagesController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $user = Auth::user();

        if ($user->is_email_ver == 0) {
            return redirect(route('verifyShow'));
        }
    }

    /**
     * Доставленные посылки
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function all_packages()
    {


        $packages = Package::where('user_id', Auth::id())->orderBy('id', 'DESC')->get();
        $declarationJson = $this->makeDeclarationJSON($packages);

        return view('pages.parcels.allpackages', [
            'packages' => $packages,
            'jsonStructure' => $declarationJson,
            'countPackages' => $this->countPackages(),
        ]);
    }

    /**
     * Посылки ожидаемые на складе
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function pending_packages()
    {
        $packages = Package::where('user_id', Auth::id())->where('status', 'pending')->orderBy('id', 'desc')->get();
        $declarationJson = $this->makeDeclarationJSON($packages);

        return view('pages.parcels.index', [
            'packages' => $packages,
            'jsonStructure' => $declarationJson,
            'countPackages' => $this->countPackages(),
        ]);
    }

    /**
     * Посылки прибывшие на склад и ожидающие упаковки
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function inwarehouse_packages(Request $request)
    {
        if ($request->isMethod('post')) {
            if ($request->has('packages2pack')) {
                $packages2pack = $request->input('packages2pack');
                foreach ($packages2pack as $id) {
                    $package = Package::find((int)$id);

                    // Проверка что посылка пренадлежит клиенту
                    if ($package->user_id != Auth::id()) {
                        Session::flash('success', 'Вы пытаетесь изменить ста    тус чужой посылки');
                        return redirect('/packages/warehouse');
                    }

                    $package->status = 'onpacking';

                    if ($package->save()) {
                        Session::flash('success', 'Посылки отправлены на упаковку');
                        return redirect('/packages/packing');
                    } else {
                        Session::flash('error', 'Невозможно отправить посылку на упаковку ошибка 7015');
                    }
                }
            }
        }

        $packages = Package::where('user_id', Auth::id())->where('status', 'instore')->orderBy('id', 'desc')->get();

        return view('pages.parcels.packages_inwarehouse', [
            'packages' => $packages,
            'jsonStructure' => $this->makeDeclarationJSON($packages),
            'countPackages' => $this->countPackages(),
        ]);
    }

    /**
     * Посылки на упаковке
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function onpacking_packages()
    {
        $packages = Package::where('user_id', Auth::id())->where('status', 'onpacking')->orderBy('id', 'desc')->get();
        return view('pages.parcels.packages_onpacking', [
            'packages' => $packages,
            'jsonStructure' => $this->makeDeclarationJSON($packages),
            'countPackages' => $this->countPackages(),
        ]);
    }

    /**
     * Отправленные посылки
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function sent_packages()
    {
        $packages = Package::where('user_id', Auth::id())->where('status', 'sent')->orderBy('id', 'desc')->get();
        return view('pages.parcels.packages_sent', [
            'packages' => $packages,
            'jsonStructure' => $this->makeDeclarationJSON($packages),
            'countPackages' => $this->countPackages(),
        ]);
    }

    public function delivered_packages()
    {
        $packages = Package::where('user_id', Auth::id())->where('status', 'delivered')->orderBy('id', 'desc')->get();
        return view('pages.parcels.packages_delivered', [
            'packages' => $packages,
            'jsonStructure' => $this->makeDeclarationJSON($packages),
            'countPackages' => $this->countPackages(),
        ]);
    }

    /**
     * Изменения статуса посылки на доставлено
     *
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function make_delivered($id)
    {
        try {
            if (($package = Package::where('user_id', Auth::id())->where('id', $id)->first()) !== null) {
                $package->status = Package::STATUS_DELIVERED;
                $package->save();
            }
        } catch (Exception $ex) {
            return redirect()->back()->with('error', $ex->getMessage());
        }

        return redirect()->route('packageDelivered')->with('success', 'Статус посылки изменен на Доставлено');
    }

    /**
     * Регистрация новой посылки
     *
     * @param Request $request
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector|\Illuminate\View\View
     */
    public function register(Request $request)
    {
        if ($request->isMethod('post')) {
            try {
                $this->handle_save($request);
                Session::flash('success', 'Ваша посылка успешно создана');
                return redirect('/packages');
            } catch (Exception $ex) {
                Session::flash('error', 'Ошибка сохранения информации о посылке: ' . $ex->getMessage());
            }
        }

        return view('addpackage');
    }

    /**
     * Prepare and save Package info with package's items
     *
     * @param $request
     * @throws Exception
     */
    public function handle_save(&$request)
    {
        $package = new Package();
        $package->user_id = Auth::id();
        $package->market_link = $request->input('market_link');
        $package->track_number = $request->input('track_number');
        $package->package_name = $request->input('package_name');
        $package->comment = $request->input('comment');

        if (!$package->save())
            throw new Exception('Невозможно создать модель Package');

        $package_item_name = $request->input('item_name');
        $package_item_amount = $request->input('amount');
        $package_item_weight = $request->input('weight');
        $package_item_price = $request->input('price');
        $i = 0;

        foreach ($package_item_name as $item_name) {
            $package_item = new PackageItems();
            $package_item->user_id = Auth::id();
            $package_item->package_id = $package->id;
            $package_item->item_name = $item_name;
            $package_item->amount = array_get($package_item_amount, $i, 0);
            $package_item->price = array_get($package_item_price, $i, 0);
            $package_item->weight = array_get($package_item_weight, $i, '');

            if (!$package_item->save())
                throw new Exception('Невозможно сохранить модель PackageItem');

            $i += 1;
        }
    }

    /**
     * Generate Declaration about packages
     *
     * @param $packages
     * @return string
     */

    private function makeDeclarationJSON(&$packages): string
    {
        $declaration_data = [];
        foreach ($packages as $package) {
            $package_items = PackageItems::where('package_id', $package->id)->get();
            $items_data = [];
            foreach ($package_items as $item) {
                $items_data[] = [
                    'package_id' => $package->id,
                    'name' => $item->item_name,
                    'amount' => $item->amount,
                    'price' => $item->price,
                    'weight' => $item->weight,
                    'date' => $item->date,
                ];
            }
            $declaration_data[$package->id] = $items_data;
        }

        return json_encode($declaration_data);
    }

    /**
     * Count packages in different statuses
     *
     * @return array
     */
    private function countPackages(): array
    {
        $itemsCounter = [
            'allPackages' => Package::where('user_id', Auth::id())->count(),
            'onPending' => Package::where('status', Package::STATUS_PENDING)->where('user_id', Auth::id())->count(),
            'inStore' => Package::where('status', Package::STATUS_INSTORE)->where('user_id', Auth::id())->count(),
            'onPacking' => Package::where('status', Package::STATUS_ONPACKING)->where('user_id', Auth::id())->count(),
            'sent' => Package::where('status', Package::STATUS_SENT)->where('user_id', Auth::id())->count(),
            'delivered' => Package::where('status', Package::STATUS_DELIVERED)->where('user_id', Auth::id())->count(),
        ];

        return $itemsCounter;
    }

    public function pay()
    {
        return view('pay');
    }

    public function addr()
    {
        return view('addpackage');
    }

    public function index()
    {
        $units = Package::where('user_id', '=', Auth::user()->id)->get();
        return view('pages.parcels.index', compact('units'));
    }
}
